'' =================================================================================================
''
''   File....... jm_dht11_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 20 DEC 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


con { io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28

  RHT = 15                                                       ' rh / temp sensor (DHT11)
  

con

  #0, TC, TF

  T_MODE = TF                                                    ' temperature mode   


obj

' main                                                           ' * master Spin cog
  time  : "jm_time"                                              '   timing and delays
  prng  : "jm_prng"                                              '   pseudo-random number generator
  io    : "jm_io_basic"                                          '   essential io
  dht11 : "jm_dht11_ez"                                          ' * DHT11 interface
  term  : "jm_fullduplexserial"                                  ' * serial IO for terminal

' * uses cog when loaded


var

  long  tavg[10]
  long  tidx

  long  havg[10]
  long  hidx


dat

  Celsius       byte    "Celsius", 0
  Fahr          byte    "Fahrenheit", 0

  TScale        word    @Celsius, @Fahr


pub main | last, now, temp, p_str

  setup

  set_pixel_char("@")                                            ' change "on" pixel for big chars

  last := 0                                                      ' allow immediate reading

  repeat
    repeat                                                       ' wait for new reading
      now := dht11.reading
    until (now <> last)
    last := now                                                  ' mark this time

    big_dec3(get_temperature, 0, 0)                              ' show temperature   
    xy_str(18, 0, @@TScale[T_MODE])   

    big_dec3(get_humidity, 0, 9)                                 ' show humidity    
    xy_str(18, 9, string("%"))
    

pub get_temperature | sum, idx

'' Reads temperature from DHT11 and returns 10-reading rolling averate

  if (T_MODE == TC)
    tavg[tidx] := dht11.tc
  else
    tavg[tidx] := dht11.tf    

  if (++tidx == 10)        
    tidx := 0

  sum := 0
  repeat idx from 0 to 9
    sum += tavg[idx]

  return sum / 10                 


pub get_humidity | sum, idx

'' Reads humidity from DHT11 and returns 10-reading rolling averate

  havg[hidx] := dht11.rh   
  if (++hidx == 10)        
    hidx := 0

  sum := 0
  repeat idx from 0 to 9
    sum += havg[idx]

  return sum / 10 

  
pub big_dec3(value, x, y) | zf, n

'' Print 3-digit decimal value with big characters (5x7) on PST
'' -- supresses leading 0s

   zf := 0                                                       ' no leading zeroes

   n := value / 100                                              ' get 100s
   if (n > 0)                                                    ' not zero?
     big_digit(x, y, n)                                          ' yes, print it
     zf := 1                                                     ' always print zeroes now
   else
     big_blank(x, y)

   n := value // 100 / 10                                        ' get 10s
   if (zf or (n > 0))
     big_digit(x+6, y, n)
   else
     big_blank(x, y)

   n := value // 10                                              ' get 1s
   big_digit(x+12, y, n)


pub xy_str(x, y, p_str)

'' Prints string a x,y on PST

  term.tx(term#GOTOXY)
  term.tx(x)
  term.tx(y)
  term.str(p_str)
  

pub setup

'' Setup IO and objects for application

  time.start                                                     ' setup timing & delays

  prng.seed(-cnt, -cnt ~> 2, $EA7_BEEF, cnt << 2, cnt)           ' seed randomizer

  io.start(0, 0)                                                 ' clear all pins (master cog)

  dht11.start(RHT, 1)                                            ' start dht11 interface * 

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial for terminal *
  time.pause(1)
  term.tx(term#CLS)                                              ' clear on reset


pub big_digit(x, y, n) | p_base, row

'' Print big digit n at x,y

  if (n < 0) or (n > 9)                                          ' validate digit
    return

  p_base := @@Digit[n]                                           ' get character address

  repeat row from 0 to 6                                         ' loop through 7 rows
    term.tx(term#GOTOXY)                                         ' move to start of row
    term.tx(x)
    term.tx(y + row)
    term.str(p_base)                                             ' print it
    p_base += strsize(p_base) + 1                                ' point to next row


pub big_blank(x, y) | row

'' Blank big character at x,y

  repeat row from 0 to 6                                         ' loop through 7 rows
    term.tx(term#GOTOXY)                                         ' move to start of row
    term.tx(x)
    term.tx(y + row)
    repeat strsize(@Digit0)                                      ' blank width of digits
      term.tx(" ")


pub set_pixel_char(ch) | n, p_base, r 

'' Set "on" pixel for big characters

  repeat n from 0 to 9                                           ' loop through all digits
    p_base := @@Digit[n]                                         ' get character address
    repeat r from 0 to Rows-1                                    ' loop through rows
      repeat strsize(p_base)                                     ' loop through chars in row
        if (byte[p_base] <> " ")
          byte[p_base] := ch
        ++p_base                                                 ' next character
      ++p_base                                                   ' next row

    
dat

  Rows          byte    7

  Digit0        byte    " XXX ", 0
                byte    "X   X", 0
                byte    "X  XX", 0
                byte    "X X X", 0   
                byte    "XX  X", 0   
                byte    "X   X", 0
                byte    " XXX ", 0

  Digit1        byte    "  X  ", 0
                byte    " XX  ", 0
                byte    "  X  ", 0
                byte    "  X  ", 0   
                byte    "  X  ", 0   
                byte    "  X  ", 0
                byte    " XXX ", 0

  Digit2        byte    " XXX ", 0
                byte    "X   X", 0
                byte    "    X", 0
                byte    "   X ", 0   
                byte    "  X  ", 0   
                byte    " X   ", 0
                byte    "XXXXX", 0
                                     
  Digit3        byte    "XXXXX", 0
                byte    "   X ", 0
                byte    "  X  ", 0
                byte    "   X ", 0   
                byte    "    X", 0   
                byte    "X   X", 0
                byte    " XXX ", 0

  Digit4        byte    "   X ", 0
                byte    "  XX ", 0
                byte    " X X ", 0
                byte    "X  X ", 0   
                byte    "XXXXX", 0   
                byte    "   X ", 0
                byte    "   X ", 0

  Digit5        byte    "XXXXX", 0
                byte    "X    ", 0
                byte    "XXXX ", 0
                byte    "    X", 0  
                byte    "    X", 0   
                byte    "X   X", 0
                byte    " XXX ", 0                

  Digit6        byte    "  XX ", 0
                byte    " X   ", 0
                byte    "X    ", 0
                byte    "XXXX ", 0   
                byte    "X   X", 0   
                byte    "X   X", 0
                byte    " XXX ", 0

  Digit7        byte    "XXXXX", 0
                byte    "    X", 0
                byte    "   X ", 0
                byte    "  X  ", 0   
                byte    " X   ", 0   
                byte    " X   ", 0
                byte    " X   ", 0

  Digit8        byte    " XXX ", 0
                byte    "X   X", 0
                byte    "X   X", 0
                byte    " XXX ", 0   
                byte    "X   X", 0   
                byte    "X   X", 0
                byte    " XXX ", 0

  Digit9        byte    " XXX ", 0
                byte    "X   X", 0
                byte    "X   X", 0
                byte    " XXXX", 0   
                byte    "    X", 0   
                byte    "   X ", 0
                byte    " XX  ", 0

  Digit         word    @Digit0, @Digit1, @Digit2, @Digit3, @Digit4
                word    @Digit5, @Digit6, @Digit7, @Digit8, @Digit9


dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}